<?php

namespace Essentials\Templates;

use Essentials\Utils\Condition;
use Essentials\Utils\Once;

class Assets {

	/**
	 * @var \Essentials\Core\Context
	 */
	private $wp_context;

	/**
	 * @var array List of styles assets
	 */
	private $styles;

	/**
	 * @var array List of scripts assets
	 */
	private $scripts;

	/**
	 * @var array CDN links dictionary
	 */
	private $cdn = array();

	/**
	 * Enqueue assets
	 *
	 * @param                          $assets object Assets to enqueue
	 * @param \Essentials\Core\Context $wp_context
	 */
	public function __construct($assets, $wp_context) {
		$this->wp_context = $wp_context;
		$this->styles     = isset($assets->css) ? $assets->css : array();
		$this->scripts    = isset($assets->js) ? $assets->js : array();

		if ($wp_context->is_admin) {
			$tag = 'admin';
		} else {
			$tag = $wp_context->is_login ? 'login' : 'wp';
		}
		$tag .= '_enqueue_scripts';

		add_action($tag, array($this, 'enqueue_scripts'), 5);

		if (!$wp_context->is_admin && !$wp_context->is_login) {
			add_action('wp_enqueue_scripts', array($this, 'enqueue_comment_reply'));
		}
	}

	/**
	 * Enqueue assets hook
	 */
	public function enqueue_scripts() {
		$wp_context = $this->wp_context;

		foreach ($this->styles as $style) {
			if (isset($style->dependency) && !Condition::check($style->dependency->type, $style->dependency->value)) {
				continue;
			}

			wp_enqueue_style(
				$style->handle,
				$wp_context->theme_uri . $style->src,
				isset($style->deps) ? $style->deps : null,
				$this->get_version($style));
		}

		foreach ($this->scripts as $script) {
			if ($script->handle === 'jquery' && !$this->wp_context->is_admin) {
				wp_deregister_script('jquery');
			}

			if (isset($script->dependency) && !Condition::check($script->dependency->type, $script->dependency->value)) {
				continue;
			}

			wp_register_script(
				$script->handle,
				(isset($script->child) && $script->child ? $wp_context->child_theme_uri : $wp_context->theme_uri) . $script->src,
				isset($script->deps) ? $script->deps : array(),
				$this->get_version($script),
				isset($script->footer) ? $script->footer : true);

			wp_enqueue_script($script->handle);

			if (isset($script->cdn) && isset($script->test)) {
				$this->cdn[$script->handle] = array($script->cdn, $script->test);

				Once::add_filter('script_loader_src', array($this, 'cdn'), 10, 2);
				Once::add_action('wp_head', array($this, 'cdn'), 10, 2);
			}
		}
	}

	/**
	 * Allow full JavaScript functionality with the comment features
	 */
	public function enqueue_comment_reply() {
		if (is_singular()) {
			wp_enqueue_script('comment-reply');
		}
	}

	/**
	 * Replace local script with CDN and local fallback
	 *
	 * @param      $src    string local script link
	 * @param null $handle script name
	 *
	 * @return string|bool False if process CDN script, otherwise do nothing and return original src
	 */
	public function cdn($src, $handle = null) {
		if (array_key_exists($handle, $this->cdn)) {
			$cdn = $this->cdn[$handle];

			echo "<script src=\"{$cdn[0]}\"></script><script>{$cdn[1]}||document.write('<script src=\"{$src}\"><\\/script>')</script>";

			return false;
		}

		return $src;
	}

	private function get_version($asset) {
		if (isset($asset->version) && $asset->version) {
			return is_string($asset->version) || is_numeric($asset->version) ? $asset->version : $this->wp_context->theme->Version;
		} else {
			return '';
		}
	}

	/**
	 * Init assets enqueue
	 *
	 * @param                          $assets object Assets to enqueue
	 * @param \Essentials\Core\Context $wp_context
	 */
	public static function init($assets, $wp_context) {
		new Assets($assets, $wp_context);
	}
}